@@ -15,6 +15,7 @@ import argparse
 
 from collections import defaultdict
 from subprocess import Popen, PIPE
+from hashlib import md5
 
 # Regexp to fetch audio formats list from `sox -h` output.
 RE_SOX_AUDIO_SUPPORT = re.compile(r'AUDIO FILE FORMATS:(.*)\n', re.MULTILINE & re.IGNORECASE)
@@ -48,6 +49,8 @@ class LetMe(object):
     _source_filename = '_letmehear.tmp.flac'
     # Part length in seconds.
     _part_length = 180
+    # Backshift in seconds.
+    _backshift = 1
     # Some lengthy shell command won't be executed on dry run.
     _dry_run = False
     # Speed ratio.
@@ -154,6 +157,13 @@ class LetMe(object):
         """Used to set output file(s) length in seconds."""
         self._part_length = seconds
 
+    def set_backshift(self, seconds):
+        """Used to set a backshift - number of seconds for every
+        part to be taken from a previous one.
+
+        """
+        self._backshift = seconds
+
     def set_speed(self, ratio):
         """Used to set output file(s) speed. Expects ratio modifier, e.g. 1.3 """
         self._speed = ratio
@@ -170,6 +180,42 @@ class LetMe(object):
         logging.debug('Sox supported audio formats: %s' % formats)
         return formats
 
+    def sox_get_sample_rates(self, files):
+        """Calculates maximum and minimum sample rates for given files.
+        Returns tuple (min_rate, max_rate, files_to_rates_dict).
+
+        """
+        min_rate = max_rate = None
+        files_to_rates_dict = {}
+
+        for file in files:
+            result = self._process_command('soxi -r "%s"' % file, PIPE)
+            rate = int(result[1][0].strip('\n'))
+            files_to_rates_dict[file] = rate
+
+            logging.debug('\nSample rate `%s` for `%s`' % (rate, file))
+
+            if max_rate is None or rate > max_rate:
+                max_rate = rate
+
+            if min_rate is None or rate < min_rate:
+                min_rate = rate
+
+        logging.debug('Sample rates: min - %s, max - %s' % (min_rate, max_rate))
+        return min_rate, max_rate, files_to_rates_dict
+
+    def get_resampled_filename(self, filepath):
+        """Returns temporary resampled file name from filepath."""
+        return 'tmp_%s.flac' % md5(filepath).hexdigest()
+
+    def sox_resample(self, file, target_rate, target_dir):
+        target_file = os.path.join(target_dir, self.get_resampled_filename(file))
+        logging.info('MUST resample "%s" to create source file. Resampling to %s ...\n      Target: %s' %
+                     (file, target_rate, target_file))
+        command = 'sox -S "%(input)s" -r %(rate)s "%(target)s"' % {'input': file, 'rate': target_rate,
+                                                                 'target': target_file}
+        self._process_command(command)
+
     def sox_create_source_file(self, files, target):
         """Creates a source file at given target path from one
         more input files.
@@ -181,14 +227,22 @@ class LetMe(object):
         options = ''
         effects = ''
 
+        files_to_precess = []
         if len(files) > 1:
             options = '--combine concatenate'
 
+        target_dir = os.path.dirname(target)
+        for file in files:
+            resampled_file = os.path.join(target_dir, self.get_resampled_filename(file))
+            if os.path.exists(resampled_file):
+                file = resampled_file
+            files_to_precess.append(file)
+
         if self._speed is not None:
             effects = 'speed %s' % self._speed
 
         command = 'sox -S --ignore-length %(options)s "%(files)s" "%(target)s" %(effects)s' % {
-            'options': options, 'files': '" "'.join(files), 'target': target, 'effects': effects}
+            'options': options, 'files': '" "'.join(files_to_precess), 'target': target, 'effects': effects}
 
         self._process_command(command)
 
@@ -204,7 +258,7 @@ class LetMe(object):
         else:
             return 1000
 
-    def sox_chop_source_audio(self, source_filename, part_length):
+    def sox_chop_source_audio(self, source_filename, part_length, backshift=0):
         """Using SoX chops source audio file into parts of given length.
         Chopping is done in such a way that every next audio part contains
         one (1) second from the previous.
@@ -217,36 +271,60 @@ class LetMe(object):
             parts_count = 1
         else:
             # Calculate audio length with one second back shift. Also known as possum formula %)
-            parts_count = int(round(wav_length / float(part_length - 1), 0))
+            parts_count = int(round(wav_length / float(part_length - backshift), 0))
         parts_count_len = len(str(parts_count))
 
-        logging.info('Chopping information:\n      Source file length: %(source)s second(s)\n      Requested part length: %(part)s second(s)\n      Parts count: %(parts_cnt)s' %
-             {'source': wav_length, 'part': part_length, 'parts_cnt': parts_count})
+        logging.info('Chopping information:\n' \
+                     '      Source file length: %(source)s second(s)\n' \
+                     '      Requested part length: %(part)s second(s)\n' \
+                     '      Backshift: %(back)s second(s)\n' \
+                     '      Parts count: %(parts_cnt)s' %
+             {'source': wav_length, 'part': part_length, 'parts_cnt': parts_count, 'back': backshift})
 
         logging.info('Starting chopping ...')
         for index in range(0, parts_count):
             start_pos = index * part_length
             if start_pos > 0:
-                # We need to shift all but the first part for one second backward
+                # We need to shift all but the first part for `backshift` seconds backward
                 # to not to loose some phrases on chopping.
-                start_pos -= index
+                start_pos -= (index * backshift)
             part_number = str(index + 1).rjust(parts_count_len, '0')
 
+            # This will strip ID3Tags from file - they are identical thus uninformative.
+            comment = '--comment ""'
+
             target = part_number
             logging.info('Writting %s.mp3 [%s/%s - %s%%] ...' % (target, int(part_number), parts_count, int(int(part_number) * 100 / parts_count)))
-            command = 'sox -V1 "%(source)s" %(target)s.mp3 trim %(start_pos)s %(length)s' % {
-                'source': source_filename, 'target': target, 'start_pos': start_pos, 'length': part_length}
+            command = 'sox -V1 "%(source)s" %(comment)s %(target)s.mp3 trim %(start_pos)s %(length)s' % {
+                'source': source_filename, 'target': target, 'start_pos': start_pos, 'length': part_length, 'comment': comment}
             self._process_command(command, PIPE)
         logging.info('Chopped.\n')
 
+    def remove_tmp_sources(self, source_filename):
+        """Removes temporary created source files."""
+        logging.info('Removing temporary files ...')
+        source_dir = os.path.dirname(source_filename)
+        if os.path.exists(source_filename):
+            os.remove(source_filename)
+        for file in os.listdir(source_dir):
+            if file.startswith('tmp_'):
+                os.remove(os.path.join(source_dir, file))
+
     def process_source_file(self, path, files, source_filename):
         """Initiates source file processing."""
         os.chdir(path)
+
+        min_rate, max_rate, files_to_rates_dict = self.sox_get_sample_rates(files)
+        if min_rate != max_rate:
+            for file in files:
+                if files_to_rates_dict[file] != max_rate:
+                    self.sox_resample(file, max_rate, os.path.dirname(source_filename))
+
         self.sox_create_source_file(files, source_filename)
         os.chdir(os.path.dirname(source_filename))
-        self.sox_chop_source_audio(source_filename, self._part_length)
-        if os.path.exists(source_filename):
-            os.remove(source_filename)
+        self.sox_chop_source_audio(source_filename, self._part_length, self._backshift)
+        self.remove_tmp_sources(source_filename)
+
 
     def hear(self, recursive=False):
         """God method that lets, as a consequence, you hear your precious
@@ -290,6 +368,7 @@ if __name__ == '__main__':
     argparser.add_argument('-r', help='Length (in seconds) for each output audio file.', action='store_true')
     argparser.add_argument('-d', help='Absolute or relative (to the current) destination path for output audio file(s).')
     argparser.add_argument('-l', help='Length (in seconds) for each output audio file.', type=int)
+    argparser.add_argument('-b', help='Backshift - number of seconds for every part to be taken from a previous one.', type=int)
     argparser.add_argument('-s', help='Speed ratio.', type=float)
     argparser.add_argument('-dry', help='Perform the dry run with no changes done to filesystem.')
     argparser.add_argument('-debug', help='Show debug messages while processing.', action='store_true')
@@ -309,6 +388,9 @@ if __name__ == '__main__':
         if parsed.l is not None:
             letme.set_part_length(parsed.l)
 
+        if parsed.b is not None:
+            letme.set_backshift(parsed.b)
+
         if parsed.s is not None:
             letme.set_speed(parsed.s)
 
